import {
  Box,
  MenuItem,
  Select,
  SelectChangeEvent,
  Stack,
  TextField,
  Typography,
} from '@mui/material'
import { SubmitHandler, useForm } from 'react-hook-form'
import React, {
  forwardRef,
  Ref,
  useEffect,
  useImperativeHandle,
  useState,
} from 'react'
import { useTranslation } from 'react-i18next'
import {
  CountryCallingCode,
  CountryCode,
  getCountries,
  getCountryCallingCode,
} from 'libphonenumber-js'
import { getAlpha2Codes, getName } from 'i18n-iso-countries'
import { GuestInviteInformation } from './guestDataForm'
import { EnteredGuestData } from './enteredGuestData'
import { GuestRegisterCallableMethods } from './guestRegisterCallableMethods'
import { isValidFnr, isValidMobilePhoneNumber } from '../../../utils'
import AuthenticationMethod from './authenticationMethod'

interface GuestRegisterProperties {
  nextHandler(guestData: EnteredGuestData): void

  guestData: GuestInviteInformation
}

/**
 * This component is the form where the guest enters missing information about himself and
 * where he can see the data the sponsor has entered and the role. The page may also
 * be populated with data from a third-party like Feide if the guest logged in using that.
 */
const GuestRegisterStep = forwardRef(
  (props: GuestRegisterProperties, ref: Ref<GuestRegisterCallableMethods>) => {
    const { i18n, t } = useTranslation(['common'])
    const { nextHandler, guestData } = props

    const [countryCode, setCountryCode] = useState<
      CountryCallingCode | undefined
    >(undefined)
    const [mobilePhone, setMobilePhone] = useState<string>('')

    const submit: SubmitHandler<EnteredGuestData> = (data) => {
      nextHandler(data)
    }

    const {
      register,
      handleSubmit,
      setValue,
      setError,
      clearErrors,
      formState: { errors },
    } = useForm<EnteredGuestData>()
    const onSubmit = handleSubmit<EnteredGuestData>(submit)

    const handleCountryCodeChange = (event: SelectChangeEvent) => {
      if (event.target.value) {
        const countryCodeType = event.target.value as CountryCode
        setCountryCode(countryCodeType)
        setValue('mobilePhoneCountry', countryCodeType)
      } else {
        setCountryCode(undefined)
      }
    }

    const handleMobilePhoneChange = (value: any) => {
      if (countryCode) {
        // The country code and the rest of the mobile number are in two fields, so cannot
        // register the field directly in form, but need to have extra logic defined
        // to combine the values before writing them to the form handling

        const phoneNumberWithCountryCode = `+${getCountryCallingCode(
          countryCode as CountryCode
        )}${value.target.value}`
        const isValidPhoneNumber = isValidMobilePhoneNumber(
          phoneNumberWithCountryCode
        )

        if (isValidPhoneNumber === true) {
          setValue('mobilePhone', value.target.value)
          clearErrors('mobilePhone')
        } else {
          setError('mobilePhone', {
            type: 'manual',
            message: isValidPhoneNumber || undefined,
          })
        }
      }

      setMobilePhone(value.target.value)
    }

    useEffect(() => {
      setCountryCode(guestData.countryForCallingCode)
      setMobilePhone(guestData.mobile_phone ? guestData.mobile_phone : '')
      setValue(
        'mobilePhoneCountry',
        guestData.countryForCallingCode ? guestData.countryForCallingCode : ''
      )
    }, [guestData])

    function doSubmit() {
      return onSubmit()
    }

    register('mobilePhone', {
      required: t<string>('validation.mobilePhoneRequired'),
    })
    register('mobilePhoneCountry')

    useImperativeHandle(ref, () => ({ doSubmit }))

    return (
      <>
        <Typography
          variant="h5"
          sx={{
            paddingTop: '1rem',
            paddingBottom: '1rem',
          }}
        >
          {t('guestRegisterWizardText.yourContactInformation')}
        </Typography>
        <Typography sx={{ paddingBottom: '2rem' }}>
          {t('guestRegisterWizardText.contactInformationDescription')}
        </Typography>
        <Box sx={{ maxWidth: '30rem' }}>
          <form onSubmit={onSubmit}>
            <Stack spacing={2}>
              <TextField
                id="firstName"
                label={t('input.firstName')}
                value={guestData.first_name}
                disabled
              />
              <TextField
                id="lastName"
                label={t('input.lastName')}
                value={guestData.last_name}
                disabled
              />

              <TextField
                id="email"
                label={t('input.email')}
                value={!guestData.email ? '' : guestData.email}
                disabled
              />

              {/* Only show the Feide ID field if the value is present */}
              {guestData.feide_id && (
                <TextField
                  id="feide_id"
                  label={t('feideId')}
                  value={guestData.feide_id}
                  disabled
                />
              )}

              <Box
                sx={{
                  display: 'flex',
                  flexDirection: 'row',
                }}
              >
                <Select
                  sx={{
                    maxHeight: '2.5rem',
                    minWidth: '5rem',
                    marginRight: '0.5rem',
                  }}
                  labelId="phone-number-select"
                  id="phone-number-select"
                  displayEmpty
                  onChange={handleCountryCodeChange}
                  value={countryCode ? countryCode.toString() : ''}
                  renderValue={(selected: any) => {
                    if (!selected) {
                      return t('input.countryCallingCode')
                    }
                    return `${getName(
                      selected,
                      i18n.language
                    )} (${getCountryCallingCode(selected as CountryCode)})`
                  }}
                  data-testid="phone-country-code-select"
                  inputProps={{
                    'data-testid': 'phone-country-code-select-inner',
                  }}
                >
                  <MenuItem disabled value="">
                    {t('input.countryCallingCode')}
                  </MenuItem>
                  {getCountries().map((country) => (
                    <MenuItem key={country} value={country}>
                      {getName(country, i18n.language)} (
                      {getCountryCallingCode(country)})
                    </MenuItem>
                  ))}
                </Select>

                <TextField
                  sx={{ flexGrow: 2 }}
                  label={t('input.mobilePhone')}
                  error={!!errors.mobilePhone}
                  value={mobilePhone}
                  helperText={errors.mobilePhone && errors.mobilePhone.message}
                  onChange={handleMobilePhoneChange}
                />
              </Box>
              {guestData.authentication_method ===
                AuthenticationMethod.Invite && (
                <>
                  {/* The guest should fill in one of national ID number or passport number */}
                  <TextField
                    id="national_id_number"
                    label={t('input.nationalIdNumber')}
                    error={!!errors.nationalIdNumber}
                    helperText={
                      errors.nationalIdNumber && errors.nationalIdNumber.message
                    }
                    {...register('nationalIdNumber', {
                      // It is not required that the national ID number be filled in, the guest may not have
                      // one, so allow empty values for the validation to pass
                      validate: (value) => isValidFnr(value, true),
                    })}
                  />

                  <TextField
                    id="passport-number-id"
                    data-testid="passport_number_input"
                    label={t('input.passportNumber')}
                    {...register('passportNumber', {
                      required: false,
                    })}
                  />

                  <Select
                    sx={{
                      maxHeight: '2.5rem',
                    }}
                    id="passport-nationality-id"
                    labelId="passport-nationality-label"
                    label={t('input.passportNationality')}
                    displayEmpty
                    defaultValue=""
                    {...register('passportNationality', {
                      required: false,
                    })}
                    renderValue={(selected: any) => {
                      if (!selected || selected.length === 0) {
                        return t('input.passportNationality')
                      }
                      return getName(selected, i18n.language)
                    }}
                  >
                    <MenuItem disabled value="">
                      {t('input.passportNationality')}
                    </MenuItem>
                    {Object.keys(getAlpha2Codes()).map((countryAlpha2Code) => (
                      <MenuItem
                        key={countryAlpha2Code}
                        value={countryAlpha2Code}
                      >
                        {getName(countryAlpha2Code, i18n.language)}
                      </MenuItem>
                    ))}
                  </Select>
                </>
              )}

              {guestData.authentication_method ===
                AuthenticationMethod.Feide && (
                <TextField
                  id="national_id_number"
                  data-testid="national_id_number_feide"
                  label={t('input.nationalIdNumber')}
                  disabled
                />
              )}

              <Typography variant="h5" sx={{ paddingTop: '1rem' }}>
                {t('guestRegisterWizardText.yourGuestPeriod')}
              </Typography>
              <Typography sx={{ paddingBottom: '1rem' }}>
                {t('guestRegisterWizardText.guestPeriodDescription')}
              </Typography>
              <TextField
                id="ou-unit"
                value={
                  i18n.language === 'en'
                    ? guestData.ou_name_en
                    : guestData.ou_name_nb
                }
                label={t('ou')}
                disabled
              />

              <TextField
                id="roleType"
                label={t('input.roleType')}
                value={
                  i18n.language === 'en'
                    ? guestData.role_name_en
                    : guestData.role_name_nb
                }
                disabled
              />

              <TextField
                id="rolePeriod"
                label={t('period')}
                value={`${guestData.role_start} - ${guestData.role_end}`}
                disabled
              />

              <TextField
                id="comment"
                label={t('input.comment')}
                multiline
                rows={5}
                value={guestData.comment}
                disabled
              />
            </Stack>
          </form>
        </Box>
      </>
    )
  }
)

export default GuestRegisterStep
