import datetime
import pytest

from greg.models import Notification, Role
from greg.signals import _queue_role_start_notification, _queue_role_end_notification


@pytest.fixture
def role_today(person, role_type_test_guest, sponsor_guy, unit_foo):
    """A test role with end and start date today."""
    role = Role.objects.create(
        person=person,
        type=role_type_test_guest,
        sponsor=sponsor_guy,
        orgunit=unit_foo,
        start_date=datetime.date.today(),
        end_date=datetime.date.today(),
    )
    return Role.objects.get(pk=role.id)


@pytest.mark.django_db
def test_delete_signal_person(person):
    """Check that a Notification is created when a Person object is deleted."""
    assert Notification.objects.count() == 1
    person.delete()
    assert Notification.objects.count() == 2


@pytest.mark.django_db
def test_delete_signal_ou(unit_foo):
    """Check that a Notification is not created when an OrganizationalUnit is deleted."""
    assert Notification.objects.count() == 0
    unit_foo.delete()
    assert Notification.objects.count() == 0


@pytest.mark.django_db
def test_queue_role_start_notification(role_today):
    """Check that a notification is produced if the role starts today"""
    assert Notification.objects.all().count() == 3
    _queue_role_start_notification(role_today.id, True)
    assert Notification.objects.all().count() == 4


@pytest.mark.django_db
def test_queue_role_end_notification(role_today):
    """Check that a notification is produced if the role ends today"""
    assert Notification.objects.all().count() == 3
    _queue_role_end_notification(role_today.id, True)
    assert Notification.objects.all().count() == 4


@pytest.mark.django_db
def test_queue_role_end_notification_wrong_date(role_today):
    """Check that a notification is produced if the role ends today"""
    role_today.end_date = datetime.date.today() - datetime.timedelta(days=2)
    role_today.save()
    assert Notification.objects.all().count() == 4
    _queue_role_end_notification(role_today.id, True)
    assert Notification.objects.all().count() == 4


@pytest.mark.django_db
def test_queue_role_end_notification_role_deleted():
    """Check that a notification is not produced if the role was deleted"""
    assert Notification.objects.all().count() == 0
    _queue_role_end_notification(10, True)
    assert Notification.objects.all().count() == 0
