from rest_framework import serializers

from greg.models import Identity, Person
from gregui.validation import (
    validate_phone_number,
    validate_norwegian_national_id_number,
)


class GuestRegisterSerializer(serializers.ModelSerializer):
    # TODO first_name and last_name set as not required to throwing an exception if they are not included in what is sent back from the frontend. It is perhaps not required that they are in the reponse from the client if the guest should be allowed to change them
    first_name = serializers.CharField(required=False)
    last_name = serializers.CharField(required=False)
    # E-mail set to not required to avoid raising exception if it is not included in input. It is not given that
    # the guest should be allowed to update it
    email = serializers.CharField(required=False)
    mobile_phone = serializers.CharField(
        required=True, validators=[validate_phone_number]
    )
    fnr = serializers.CharField(
        required=False, validators=[validate_norwegian_national_id_number]
    )
    passport = serializers.CharField(required=False)

    def update(self, instance, validated_data):
        mobile_phone = validated_data.pop("mobile_phone")

        if "email" in validated_data:
            email = validated_data.pop("email")
            create_identity_or_update(
                Identity.IdentityType.PRIVATE_EMAIL, email, instance
            )

        if not instance.private_mobile:
            create_identity_or_update(
                Identity.IdentityType.PRIVATE_MOBILE_NUMBER, mobile_phone, instance
            )

        if "fnr" in validated_data:
            fnr = validated_data.pop("fnr")
            create_identity_or_update(
                Identity.IdentityType.NORWEGIAN_NATIONAL_ID_NUMBER, fnr, instance
            )

        if "passport" in validated_data:
            passport = validated_data.pop("passport")
            if not instance.passport:
                Identity.objects.create(
                    person=instance,
                    type=Identity.IdentityType.PASSPORT_NUMBER,
                    value=passport,
                )
            else:
                passport_existing = instance.passport
                passport_existing.value = passport
                passport_existing.save()

        # TODO: we only want to allow changing the name if we don't have one
        #       from a reliable source (Feide/KORR)
        # TODO Comment back in after it is decided if name updates are allowed
        # instance.first_name = validated_data["first_name"]
        # instance.last_name = validated_data["last_name"]

        return instance

    class Meta:
        model = Person
        fields = (
            "id",
            "first_name",
            "last_name",
            "email",
            "mobile_phone",
            "fnr",
            "passport",
        )
        read_only_fields = ("id",)


def create_identity_or_update(
    identity_type: Identity.IdentityType, value: str, person: Person
):
    existing_identity = person.identities.filter(type=identity_type).first()
    if not existing_identity:
        Identity.objects.create(
            person=person,
            type=identity_type,
            value=value,
        )
    else:
        existing_identity.value = value
        existing_identity.save()
